﻿
// ---------------  ARRANGE TRACKS ------------------------------
// returns an array of all  tracks
function getTracks(context)
{
    var tracks = new Array();
    var trackList = context.mainTrackList;

    for (i = 0; i < trackList.numTracks; i++)
    {
        var track = trackList.getTrack(i);

        // avoid duplicating array entries
        if (tracks.indexOf(track) < 0)
        {
            tracks.push(track);
        }
    }
    return tracks;
}

// ---------------------------------------------------------------
// returns an array of selected tracks
function getSelectedTracks(context)
{
    var tracks = new Array();
    var trackList = context.mainTrackList;

    for (i = 0; i < trackList.numSelectedTracks; i++)
    {
        var track = trackList.getSelectedTrack(i);
        
        // avoid possibly duplicating array entries
        if (tracks.indexOf(track) < 0)
        {
            tracks.push(track);
        }
    }
    
    return tracks;
}

// --------------------------------  MIXER CHANNELS ----------------------------------------
// return master bus object
function getMasterBus(context)
{
    return context.functions.root.environment.find("MixerConsole").getChannelList(3).getChannel(0);
}

// return mixer channels
function getChannels(context)
{   
    var console = context.functions.root.environment.find("MixerConsole");
    var channelList = console.getChannelList(1);
    var channels = new Array();

            for (i=0; i < channelList.numChannels; i++)
            {
                channels.push( channelList.getChannel(i) )
            }
    
    return channels;
}

// return selected mixer channels
function getSellectedChannels(context)
{   
    var console = context.functions.root.environment.find("MixerConsole");
    var channelList = console.getChannelList(1);
    var channels = new Array();

            for (i=0; i < channelList.numSelectedChannels; i++)
            {
                channels.push( channelList.getSelectedChannel(i) )
            }
    
    return channels;
}

// set selected pans to unity
function resetFaders(context)
{
    var console = context.functions.root.environment.find("MixerConsole");
    var channelList = console.getChannelList(1);

            for (i=0; i < channelList.numSelectedChannels; i++)
            {
                var channel = channelList.getSelectedChannel(i);
                channel.volumne = 1;
            }
}

// set selected pans to center
function resetPans(context)
{
    var console = context.functions.root.environment.find("MixerConsole");
    var channelList = console.getChannelList(1);

    for (i=0; i < channelList.numSelectedChannels; i++)
    {
        var channel = channelList.getSelectedChannel(i);
        channel.pan = 0.5;
    }
}

// toggle stereo panning on selected channel pairs

// trim or gain any fader level by any amount
function faderLevel(context,value)
{
    var console = context.functions.root.environment.find("MixerConsole");
    var channelList = console.getChannelList(1);
    for (i=0; i < channelList.numSelectedChannels; i++)
    {  
        var channel = channelList.getSelectedChannel(i);
        var curGain =  floatToDb( channel.volume )

        curGain = curGain + (value);

        channel.volume= dbToFloat(curGain);
    }
}

// ---------------------------- MESSAGES -------------------------------------------------

// shortcut to call alert box
function alert(msg)
{
    Host.GUI.alert(msg.toString());
}

// shortcut to call ask message
function ask(msg)
{
    var result = Host.GUI.ask(msg) // != Host.GUI.Constants.kYes
    return result;
}

// ------------------- FILE I/O -----------------------------------------------------

// batch write text to a simple text file
function writeTextFile(folder, filename, text)
{
    var path = Host.Url("local://$USERCONTENT/" + folder + "/" + filename + ".txt");
    let textFile = Host.IO.createTextFile(path);
    if (textFile)
    {
        textFile.writeLine(text);
        textFile.close();
    }
}


// read simple text file line by line into an array
function readTextFile(folder, filename)
{
    var path = Host.Url("local://$USERCONTENT/" + folder + "/" + filename + ".txt");
    let textFile = Host.IO.openTextFile(path);

    if (textFile)
    {
        var fileArray = new Array();
        while (!textFile.endOfStream)
        {
            fileArray.push(textFile.readLine());
        }

        textFile.close();
    }
    return fileArray;
}

// ---------------------------------------------------------------------------------------

// convert the hex color value to integer
function getColorVal (hexcolor)
{
    let value = parseInt(hexcolor, 16)
    let r = (value >> 16) & 0xff;
    let g = (value >> 8) & 0xff;
    let b = value & 0xff;
    return ((b << 16) | (g << 8) | r) | 0xff000000;
}


// ---------------- MATH CONVERSIONS FOR FADER ----------------------------------------------

function dbToFloat (value)
{
   var result =  (Math.pow(10, parseFloat(value / 20)))
    return result;
}

function floatToDb (value)
{
   var result =  ( Math.log (  parseFloat(value) ) / Math.LN10 ) *20
    return result;
}

// --------------  STORE A NEW VERSION BACKUP ---------------------------------------------
function saveNewVersion(name)
{
    Host.GUI.Commands.interpretCommand("File", "Save New Version", false, Host.Attributes(["Description", name]));
}

// ---------- GET THE SONG FOLDER PATH ------------------------------------------

function getSongFolder (fileName)
{
    var docManager = Host.Objects.getObjectByUrl ("://studioapp/DocumentManager");
    var document = docManager.activeDocument;
    var path = document.path;
    path.ascend ();		
    path.descend (fileName);
    return path;
}

// ---------- GET SELECTE ARRANGE EVENTS ----------------------------------------

function getEvents(context)
{
    var iterator = context.iterator
    var events = new Array();

        while(!iterator.done())
        {
            var event = iterator.next();
            events.push(event);
        }
}

// ----------- FIRE AN ACTION ----------------------------------------------------

function command(cat, action, arg, val, arg2, val2)
{
    if (arg == undefined) {arg = ""}
    if (val == undefined) {val = ""}
    if (arg2 == undefined) {arg2 = ""}
    if (val2 == undefined) {val2 = ""}

    Host.GUI.Commands.interpretCommand(cat, action, false, Host.Attributes( [arg, val, arg2, val2] ) );
}
// ------------- PARSE OBJECT MODEL ------------------------------------

function getAllPropertyNames(obj) {
   var props = [];
   do {
       props = props.concat(Object.getOwnPropertyNames(obj));
   } while (obj = Object.getPrototypeOf(obj));

   var result = props.join('\r\n');

   //exportItems(result);
   Host.GUI.alert(String(result));
}

