// save a new version backup
function saveNewVersion(name)
{
    Host.GUI.Commands.interpretCommand("File", "Save New Version", false, Host.Attributes(["Description", name]));
}

// -----------------------------------------------------------------------------------------------------------------------------------

// track indexes change in real time when removing tracks so putting them into an array 
// allows addressing them directly and not by index when doing things like removing tracks

// returns an array of all visible tracks
function getTracks(context, selected)
{
    var tracks = new Array();
    var trackList = context.mainTrackList;

    for (i = 0; i < trackList.numTracks; i++)
    {
        var track = trackList.getTrack(i);

        // avoid duplicating array entries
        if (tracks.indexOf(track) < 0)
        {
            tracks.push(track);
        }
    }
    return tracks;
}

// ---------------------------------------------------------------------------------------

// returns an array of  tracks
function getTracks(context,selected)
{
    var tracks = new Array();
    var trackList = context.mainTrackList;

    switch (selected)
    {
        case true:
            for (i = 0; i < trackList.numSelectedTracks; i++)
            {
                var track = trackList.getSelectedTrack(i);
        
                // avoid possibly duplicating array entries
                if (tracks.indexOf(track) < 0)
                {
                    tracks.push(track);
                }
            }
        break;
        
        case false:
            for (i = 0; i < trackList.numTracks; i++)
            {
                var track = trackList.getTrack(i);
        
                // avoid duplicating array entries
                if (tracks.indexOf(track) < 0)
                {
                    tracks.push(track);
                }
            }
        break;
    }


   
    return tracks;
}

// ---------------------------------------------------------------------------------------
// mixer channels, 1=normal, 2=sub, 3=master
function getChannels(context, type, selected)
{   
    // return master bus 
    if (type == 3)
    {
       return context.functions.root.environment.find("MixerConsole").getChannelList(3).getChannel(0);
    }

    var console = context.functions.root.environment.find("MixerConsole");
    var channelList = console.getChannelList(type);
    var channels = new Array();

    switch (selected)
    {
        case true:
            for (i=0; i < channelList.numSelectedChannels; i++)
            {
                channels.push( channelList.getSelectedChannel(i) ) 
            }
        break;
        
        case false:
            for (i=0; i < channelList.numChannels; i++)
            {
                channels.push( channelList.getChannel(i) )
            }
        break;
    }
    return channels;
}

// ---------------------------------------------------------------------------------------

// lowercase all letters and cap the first letter in every word
function capWords(trackName)
{
    // make text lower case
    trackName = trackName.toLowerCase();

    // split words into an array
    var words = trackName.split(' ');

    // create an array for the letters
    var chars = [];

    for (i = 0; i < words.length; i++)
    {
        // capitalize the first letter in every word
        chars.push(words[i].charAt(0).toUpperCase() + words[i].slice(1));
    }

    // put humpy back together and return him
    return chars.join(' ');
}

// ---------------------------------------------------------------------------------------

// shortcut to call alert box
function alert(msg)
{
    Host.GUI.alert(msg);
}

// ---------------------------------------------------------------------------------------

// shortcut to call ask message
function ask(msg)
{
    var result = Host.GUI.ask(msg) // != Host.GUI.Constants.kYes
    return result;
}

// ---------------------------------------------------------------------------------------

// batch write text to a simple text file
function writeTextFile(folder, filename, text)
{
    var path = Host.Url("local://$USERCONTENT/" + folder + "/" + filename + ".txt");
    let textFile = Host.IO.createTextFile(path);
    if (textFile)
    {
        textFile.writeLine(text);
        textFile.close();
    }
}

// ---------------------------------------------------------------------------------------

// read simple text file line by line into an array
function readTextFile(folder, filename)
{
    var path = Host.Url("local://$USERCONTENT/" + folder + "/" + filename + ".txt");
    let textFile = Host.IO.openTextFile(path);

    if (textFile)
    {
        var fileArray = new Array();
        while (!textFile.endOfStream)
        {
            fileArray.push(textFile.readLine());
        }

        textFile.close();
    }
    return fileArray;
}

// ---------------------------------------------------------------------------------------

// read the fade settings from the fadeSettings.txt file in the User Data folder
function getFadeSettings()
{
    var fade = { length: 0, type: 0, bend: 0 }

    // try to load from file, if fail, use default settings
    try
    {
        var fadeSettings = readTextFile(scriptTitle, "fadeSettings")
        var data = fadeSettings[0].split("|");
        fade.length = Number(data[0]);
        fade.type = Number(data[1]);
    }
    catch (err)
    {
        // default settings if nothing has been saved yet
        fade.length = 0.005;
        fadeType = 1;
    }

    // set a good default bend curve based on the fade type
    if (fade.type == 2) { fade.bend = 0.15 } else { fade.bend = 0.66 }

    return (fade)
}

// ---------------------------------------------------------------------------------------

// convert the hex color value to integer
function getColorVal (hexcolor)
{
    let value = parseInt(hexcolor, 16)
    let r = (value >> 16) & 0xff;
    let g = (value >> 8) & 0xff;
    let b = value & 0xff;
    return ((b << 16) | (g << 8) | r) | 0xff000000;
}

// ---------------------------------------------------------------------------------------

function dbToFloat (value)
{
   var result =  (Math.pow(10, parseFloat(value / 20)))
    return result;
}

function floatToDb (value)
{
   var result =  ( Math.log (  parseFloat(value) ) / Math.LN10 ) *20
    return result;
}